'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

var _ci = require('./ci360.utils');

function _toConsumableArray(arr) { if (Array.isArray(arr)) { for (var i = 0, arr2 = Array(arr.length); i < arr.length; i++) { arr2[i] = arr[i]; } return arr2; } else { return Array.from(arr); } }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

var CI360Viewer = function () {
  function CI360Viewer(container, fullScreen, ratio) {
    _classCallCheck(this, CI360Viewer);

    this.container = container;
    this.activeImage = 1;
    this.previousActiveImage = 1;
    this.movementStart = 0;
    this.isClicked = false;
    this.loadedImages = 0;
    this.imagesLoaded = false;
    this.reversed = false;
    this.fullScreenView = !!fullScreen;
    this.ratio = ratio;

    this.init(container);
  }

  _createClass(CI360Viewer, [{
    key: 'mousedown',
    value: function mousedown(event) {
      event.preventDefault();

      if (!this.imagesLoaded) return;

      if (this.glass) {
        this.closeMagnifier();
      }

      if (this.view360Icon) {
        this.remove360ViewIcon();
      }

      if (this.autoplay) {
        this.stop();
        this.autoplay = false;
      }

      this.movementStart = event.pageX;
      this.isClicked = true;
      this.container.style.cursor = 'grabbing';
    }
  }, {
    key: 'mouseup',
    value: function mouseup() {
      if (!this.imagesLoaded) return;

      this.movementStart = 0;
      this.isClicked = false;
      this.container.style.cursor = 'grab';

      if (this.bottomCircle) {
        this.show360ViewCircleIcon();
      }
    }
  }, {
    key: 'mousemove',
    value: function mousemove(event) {
      if (!this.isClicked || !this.imagesLoaded) return;

      this.onMove(event.pageX);
    }
  }, {
    key: 'touchstart',
    value: function touchstart(event) {
      if (!this.imagesLoaded) return;

      if (this.glass) {
        this.closeMagnifier();
      }

      if (this.view360Icon) {
        this.remove360ViewIcon();
      }

      if (this.autoplay) {
        this.stop();
        this.autoplay = false;
      }

      event.preventDefault();
      this.movementStart = event.touches[0].clientX;
      this.isClicked = true;
    }
  }, {
    key: 'touchend',
    value: function touchend() {
      if (!this.imagesLoaded) return;

      this.movementStart = 0;
      this.isClicked = false;

      if (this.bottomCircle) this.show360ViewCircleIcon();
    }
  }, {
    key: 'touchmove',
    value: function touchmove(event) {
      if (!this.isClicked || !this.imagesLoaded) return;

      this.onMove(event.touches[0].clientX);
    }
  }, {
    key: 'keydownGeneral',
    value: function keydownGeneral() {
      if (!this.imagesLoaded) return;

      if (this.glass) {
        this.closeMagnifier();
      }
    }
  }, {
    key: 'keydown',
    value: function keydown(event) {
      var _this = this;

      if (!this.imagesLoaded) return;

      if (this.glass) {
        this.closeMagnifier();
      }

      if ([37, 39].indexOf(event.keyCode) !== -1) {
        var reversed = 37 === event.keyCode;

        reversed ? this.prev() : this.next();

        if (this.bottomCircle) {
          this.hide360ViewCircleIcon();
        }

        if (this.view360Icon) {
          this.remove360ViewIcon();
        }

        if (this.autoplay) {
          this.stop();
          this.autoplay = false;
        }

        this.loopTimeoutId = window.setTimeout(function () {
          _this.loop(reversed);
        }, 300);
      }
    }
  }, {
    key: 'keyup',
    value: function keyup(event) {
      if (!this.imagesLoaded) return;

      if ([37, 39].indexOf(event.keyCode) !== -1) {
        if (this.bottomCircle) this.show360ViewCircleIcon();
        window.clearTimeout(this.loopTimeoutId);
      }
    }
  }, {
    key: 'onMove',
    value: function onMove(pageX) {
      if (pageX - this.movementStart > this.speedFactor) {
        var itemsSkipped = Math.floor((event.pageX - this.movementStart) / this.speedFactor) || 1;

        this.movementStart = pageX;
        this.previousActiveImage = this.activeImage;

        this.activeImage = (this.activeImage + itemsSkipped) % this.amount || 1;

        if (this.bottomCircle) this.hide360ViewCircleIcon();
        this.update(this.activeImage);
      } else if (pageX - this.movementStart <= -this.speedFactor) {
        var _itemsSkipped = Math.abs(Math.floor((pageX - this.movementStart) / this.speedFactor)) || 1;

        this.movementStart = pageX;
        this.previousActiveImage = this.activeImage;

        if (this.activeImage - _itemsSkipped < 1) {
          this.activeImage = this.amount + (this.activeImage - _itemsSkipped);
        } else {
          this.activeImage = this.activeImage - _itemsSkipped;
        }

        if (this.bottomCircle) this.hide360ViewCircleIcon();
        this.update(this.activeImage);
      }
    }
  }, {
    key: 'loop',
    value: function loop(reversed) {
      reversed ? this.prev() : this.next();
    }
  }, {
    key: 'next',
    value: function next() {
      this.previousActiveImage = this.activeImage;
      this.activeImage = this.activeImage % this.amount + 1;

      this.update(this.activeImage);
    }
  }, {
    key: 'prev',
    value: function prev() {
      this.previousActiveImage = this.activeImage;
      if (this.activeImage - 1 === 0) {
        this.activeImage = this.amount;
      } else {
        this.activeImage -= 1;
      }

      this.update(this.activeImage);
    }
  }, {
    key: 'update',
    value: function update(activeImage) {
      var prevImage = this.container.children[this.previousActiveImage - 1];
      var nextImage = this.container.children[activeImage - 1];

      prevImage.style.display = 'none';
      nextImage.style.display = 'block';
    }
  }, {
    key: 'onImageLoad',
    value: function onImageLoad() {
      this.loadedImages += 1;

      var percentage = Math.round(this.loadedImages / this.amount * 100);

      if (this.loader) {
        this.loader.style.width = percentage + '%';
      }

      if (this.view360Icon) {
        this.view360Icon.innerText = percentage + '%';
      }

      if (this.loadedImages === this.amount) {
        this.imagesLoaded = true;
        this.container.style.cursor = 'grab';
        this.removeLoader();

        if (!this.fullScreenView) {
          this.speedFactor = Math.floor(36 / this.amount * 25 * this.container.offsetWidth / 1500) || 1;
        } else {
          var containerRatio = this.container.offsetHeight / this.container.offsetWidth;
          var imageOffsetWidth = this.container.offsetWidth;

          if (this.ratio > containerRatio) {
            imageOffsetWidth = this.container.offsetHeight / this.ratio;
          }

          this.speedFactor = Math.floor(36 / this.amount * 25 * imageOffsetWidth / 1500) || 1;
        }

        if (this.autoplay) {
          this.play();
        }

        if (this.view360Icon) {
          this.view360Icon.innerText = '';
          (0, _ci.setView360Icon)(this.view360Icon);
        }
      } else if (this.loadedImages === 1) {
        this.add360ViewIcon();

        if (this.lazyload) {
          [].slice.call(this.container.children).filter(function (image) {
            return image.nodeName === 'IMG';
          }).forEach(function (image, index) {
            if (index === 0) return;

            var dataSrc = image.getAttribute('data-src');

            if (dataSrc) {
              image.src = image.getAttribute('data-src');
            }
          });
        }

        if (this.ratio) {
          this.container.style.minHeight = 'auto';
        }

        if (this.magnifier && !this.fullScreenView) {
          this.addMagnifier();
        }

        if (this.boxShadow && !this.fullScreenView) {
          this.addBoxShadow();
        }

        if (this.bottomCircle && !this.fullScreenView) {
          this.add360ViewCircleIcon();
        }

        if (this.fullScreen && !this.fullScreenView) {
          this.addFullScreenIcon();
        } else if (this.fullScreenView) {
          this.addCloseFullScreenView();
        }
      }
    }
  }, {
    key: 'addCloseFullScreenView',
    value: function addCloseFullScreenView() {
      var closeFullScreenIcon = document.createElement('div');

      (0, _ci.setCloseFullScreenViewStyles)(closeFullScreenIcon);

      closeFullScreenIcon.onclick = this.closeFullScreenModal.bind(this);

      this.container.appendChild(closeFullScreenIcon);
    }
  }, {
    key: 'add360ViewIcon',
    value: function add360ViewIcon() {
      var view360Icon = document.createElement('div');

      (0, _ci.set360ViewIconStyles)(view360Icon);

      view360Icon.innerText = '0%';

      this.view360Icon = view360Icon;
      this.container.appendChild(view360Icon);
    }
  }, {
    key: 'addFullScreenIcon',
    value: function addFullScreenIcon() {
      var fullScreenIcon = document.createElement('div');

      (0, _ci.setFullScreenIconStyles)(fullScreenIcon);

      fullScreenIcon.onclick = this.openFullScreenModal.bind(this);

      this.container.appendChild(fullScreenIcon);
    }
  }, {
    key: 'addMagnifier',
    value: function addMagnifier() {
      var magnifyIcon = document.createElement('div');

      (0, _ci.setMagnifyIconStyles)(magnifyIcon, this.fullScreen);

      magnifyIcon.onclick = this.magnify.bind(this);

      this.container.appendChild(magnifyIcon);
    }
  }, {
    key: 'magnify',
    value: function magnify() {
      var _this2 = this;

      var src = '' + this.folder + this.filename.replace('{index}', this.activeImage);
      var image = new Image();

      image.src = src;
      image.onload = function () {
        if (_this2.glass) {
          _this2.glass.style.cursor = 'none';
        }
      };

      this.glass = document.createElement('div');
      this.container.style.overflow = 'hidden';
      (0, _ci.magnify)(this.container, this.container.children[this.activeImage], src, this.glass, this.magnifier || 3);
    }
  }, {
    key: 'closeMagnifier',
    value: function closeMagnifier() {
      if (!this.glass) return;

      this.container.style.overflow = 'visible';
      this.container.removeChild(this.glass);
      this.glass = null;
    }
  }, {
    key: 'openFullScreenModal',
    value: function openFullScreenModal() {
      var fullScreenModal = document.createElement('div');

      (0, _ci.setFullScreenModalStyles)(fullScreenModal);

      var fullScreenContainer = this.container.cloneNode();
      var image = this.container.children[0];
      var ratio = image.height / image.width;

      fullScreenContainer.style.height = '100%';
      fullScreenContainer.style.maxHeight = '100%';

      fullScreenModal.appendChild(fullScreenContainer);

      window.document.body.appendChild(fullScreenModal);

      new CI360Viewer(fullScreenContainer, true, ratio);
    }
  }, {
    key: 'closeFullScreenModal',
    value: function closeFullScreenModal() {
      document.body.removeChild(this.container.parentNode);
    }
  }, {
    key: 'add360ViewCircleIcon',
    value: function add360ViewCircleIcon() {
      var view360CircleIcon = new Image();

      (0, _ci.set360ViewCircleIconStyles)(view360CircleIcon, this.bottomCircleOffset);

      this.view360CircleIcon = view360CircleIcon;
      this.container.appendChild(view360CircleIcon);
    }
  }, {
    key: 'hide360ViewCircleIcon',
    value: function hide360ViewCircleIcon() {
      if (!this.view360CircleIcon) return;

      this.view360CircleIcon.style.opacity = 0;
    }
  }, {
    key: 'show360ViewCircleIcon',
    value: function show360ViewCircleIcon() {
      if (!this.view360CircleIcon) return;

      this.view360CircleIcon.style.opacity = 1;
    }
  }, {
    key: 'remove360ViewCircleIcon',
    value: function remove360ViewCircleIcon() {
      if (!this.view360CircleIcon) return;

      this.container.removeChild(this.view360CircleIcon);
      this.view360CircleIcon = null;
    }
  }, {
    key: 'addLoader',
    value: function addLoader() {
      var loader = document.createElement('div');

      (0, _ci.setLoaderStyles)(loader);

      this.loader = loader;
      this.container.appendChild(loader);
    }
  }, {
    key: 'addBoxShadow',
    value: function addBoxShadow() {
      var boxShadow = document.createElement('div');

      (0, _ci.setBoxShadowStyles)(boxShadow, this.boxShadow);

      this.container.appendChild(boxShadow);
    }
  }, {
    key: 'removeLoader',
    value: function removeLoader() {
      if (!this.loader) return;

      this.container.removeChild(this.loader);
      this.loader = null;
    }
  }, {
    key: 'remove360ViewIcon',
    value: function remove360ViewIcon() {
      if (!this.view360Icon) return;

      this.container.removeChild(this.view360Icon);
      this.view360Icon = null;
    }
  }, {
    key: 'play',
    value: function play() {
      var _this3 = this;

      if (this.bottomCircle) this.hide360ViewCircleIcon();
      this.remove360ViewIcon();

      this.loopTimeoutId = window.setInterval(function () {
        _this3.loop(_this3.reversed);
      }, this.speed * 36 / this.amount);
    }
  }, {
    key: 'stop',
    value: function stop() {
      if (this.bottomCircle) this.show360ViewCircleIcon();
      window.clearTimeout(this.loopTimeoutId);
    }
  }, {
    key: 'init',
    value: function init(container) {
      var _this4 = this;

      var _get360ViewProps = (0, _ci.get360ViewProps)(container),
          folder = _get360ViewProps.folder,
          filename = _get360ViewProps.filename,
          amount = _get360ViewProps.amount,
          _get360ViewProps$drag = _get360ViewProps.draggable,
          draggable = _get360ViewProps$drag === undefined ? true : _get360ViewProps$drag,
          _get360ViewProps$swip = _get360ViewProps.swipeable,
          swipeable = _get360ViewProps$swip === undefined ? true : _get360ViewProps$swip,
          keys = _get360ViewProps.keys,
          bottomCircle = _get360ViewProps.bottomCircle,
          bottomCircleOffset = _get360ViewProps.bottomCircleOffset,
          boxShadow = _get360ViewProps.boxShadow,
          autoplay = _get360ViewProps.autoplay,
          speed = _get360ViewProps.speed,
          autoplayReverse = _get360ViewProps.autoplayReverse,
          fullScreen = _get360ViewProps.fullScreen,
          magnifier = _get360ViewProps.magnifier,
          ratio = _get360ViewProps.ratio,
          responsive = _get360ViewProps.responsive,
          ciToken = _get360ViewProps.ciToken,
          ciSize = _get360ViewProps.ciSize,
          ciOperation = _get360ViewProps.ciOperation,
          ciFilters = _get360ViewProps.ciFilters,
          lazyload = _get360ViewProps.lazyload,
          lazySelector = _get360ViewProps.lazySelector;

      this.addLoader();

      [].concat(_toConsumableArray(new Array(amount))).map(function (item, index) {
        var image = new Image();
        var src = '' + folder + filename.replace('{index}', index + 1);

        if (responsive) {
          var imageOffsetWidth = container.offsetWidth;

          if (_this4.fullScreenView) {
            var containerRatio = _this4.container.offsetHeight / _this4.container.offsetWidth;

            if (_this4.ratio > containerRatio) {
              imageOffsetWidth = _this4.container.offsetHeight / _this4.ratio;
            }
          }

          var ciSizeNext = (0, _ci.getSizeAccordingToPixelRatio)(ciSize || (0, _ci.getResponsiveWidthOfContainer)(imageOffsetWidth));

          src = 'https://' + ciToken + '.cloudimg.io/' + ciOperation + '/' + ciSizeNext + '/' + ciFilters + '/' + src;
        }

        if (lazyload) {
          image.setAttribute('data-src', src);
          image.className = image.className.length ? image.className + (' ' + lazySelector) : lazySelector;
        } else {
          image.src = src;
        }

        image.style.height = 'auto';
        image.style.width = '100%';
        image.style.display = index === 0 ? 'block' : 'none';
        image.onload = _this4.onImageLoad.bind(_this4);
        image.onerror = _this4.onImageLoad.bind(_this4);

        _this4.folder = folder;
        _this4.filename = filename;
        _this4.amount = amount;
        _this4.bottomCircle = bottomCircle;
        _this4.bottomCircleOffset = bottomCircleOffset;
        _this4.boxShadow = boxShadow;
        _this4.autoplay = autoplay;
        _this4.speed = speed;
        _this4.reversed = autoplayReverse;
        _this4.fullScreen = fullScreen;
        _this4.magnifier = magnifier;
        _this4.lazyload = lazyload;

        container.style.position = 'relative';
        container.style.cursor = 'wait';

        if (_this4.fullScreenView) {
          var imageBackground = document.createElement('div');
          (0, _ci.setFullScreenImageStyles)(imageBackground, src, index);
          container.appendChild(imageBackground);
        } else {
          _this4.ratio = ratio;

          if (ratio) {
            container.style.minHeight = container.offsetWidth * ratio + 'px';
          }

          container.appendChild(image);
        }
      });

      if (draggable) {
        container.addEventListener('mousedown', this.mousedown.bind(this));
        container.addEventListener('mouseup', this.mouseup.bind(this));
        container.addEventListener('mousemove', this.mousemove.bind(this));
      }

      if (swipeable) {
        container.addEventListener('touchstart', this.touchstart.bind(this));
        container.addEventListener('touchend', this.touchend.bind(this));
        container.addEventListener('touchmove', this.touchmove.bind(this));
      }

      if (keys) {
        document.addEventListener('keydown', this.keydown.bind(this));
        document.addEventListener('keyup', this.keyup.bind(this));
      } else {
        document.addEventListener('keydown', this.keydownGeneral.bind(this));
      }
    }
  }]);

  return CI360Viewer;
}();

exports.default = CI360Viewer;